@@ -55,8 +55,8 @@ class TestAptCache(unittest.TestCase):
         # that is possible and does not crash
         for pkg in cache:
             if pkg.candidate:
-                for or_dep in pkg.candidate.dependencies:
-                    for dep in or_dep.or_dependencies:
+                for or_deps in pkg.candidate.dependencies:
+                    for dep in or_deps:
                         self.assertTrue(dep.name)
                         self.assertTrue(isinstance(dep.relation, str))
                         self.assertTrue(dep.pre_depend in (True, False))
@@ -74,6 +74,9 @@ class TestAptCache(unittest.TestCase):
         apt.apt_pkg.config.set("Apt::architecture", "i386")
         cache = apt.Cache(rootdir="./data/test-provides/")
         cache.open()
+        if len(cache) == 0:
+            logging.warn("skipping test_get_provided_packages, cache empty?!?")
+            return
         # a true virtual pkg
         l = cache.get_providing_packages("mail-transport-agent")
         self.assertTrue(len(l) > 0)
@@ -84,6 +87,9 @@ class TestAptCache(unittest.TestCase):
         apt.apt_pkg.config.set("Apt::architecture", "i386")
         # create highlevel cache and get the lowlevel one from it
         highlevel_cache = apt.Cache(rootdir="./data/test-provides")
+        if len(highlevel_cache) == 0:
+            logging.warn("skipping test_log_level_pkg_provides, cache empty?!?")
+            return
         # low level cache provides list of the pkg
         cache = highlevel_cache._cache
         l = cache["mail-transport-agent"].provides_list
@@ -181,5 +187,21 @@ class TestAptCache(unittest.TestCase):
         apt_pkg.config.set("dir::etc::sourcelist", old_source_list)
         apt_pkg.config.set("dir::etc::sourceparts", old_source_parts)
 
+    def test_package_cmp(self):
+        cache = apt.Cache(rootdir="/")
+        l = []
+        l.append(cache["libc6"])
+        l.append(cache["python3"])
+        l.append(cache["apt"])
+        l.sort()
+        self.assertEqual([p.name for p in l],
+                         ["apt", "libc6", "python3"])
+
+    def test_get_architectures(self):
+        main_arch = apt.apt_pkg.config.get("APT::Architecture")
+        arches = apt_pkg.get_architectures()
+        self.assertTrue(main_arch in arches)
+
+
 if __name__ == "__main__":
     unittest.main()
